@@ -1,8 +1,8 @@
 use crate::{
-    input::user::KeyBinding,
-    markdown::elements::SnippetLanguage,
-    media::{emulator::TerminalEmulator, kitty::KittyMode},
     GraphicsMode,
+    input::user::KeyBinding,
+    media::{emulator::TerminalEmulator, kitty::KittyMode},
+    processing::code::SnippetLanguage,
 };
 use clap::ValueEnum;
 use schemars::JsonSchema;
@@ -132,6 +132,10 @@ pub struct SnippetConfig {
     #[serde(default)]
     pub exec: SnippetExecConfig,
 
+    /// The properties for snippet execution.
+    #[serde(default)]
+    pub exec_replace: SnippetExecReplaceConfig,
+
     /// The properties for snippet auto rendering.
     #[serde(default)]
     pub render: SnippetRenderConfig,
@@ -148,6 +152,14 @@ pub struct SnippetExecConfig {
     pub custom: BTreeMap<SnippetLanguage, LanguageSnippetExecutionConfig>,
 }
 
+#[derive(Clone, Debug, Default, Deserialize, JsonSchema)]
+#[serde(deny_unknown_fields)]
+pub struct SnippetExecReplaceConfig {
+    /// Whether to enable snippet replace-executions, which automatically run code snippets without
+    /// the user's intervention.
+    pub enable: bool,
+}
+
 #[derive(Clone, Debug, Deserialize, JsonSchema)]
 #[serde(deny_unknown_fields)]
 pub struct SnippetRenderConfig {
@@ -214,6 +226,9 @@ pub struct LanguageSnippetExecutionConfig {
 
     /// The commands to be run when executing snippets for this programming language.
     pub commands: Vec<Vec<String>>,
+
+    /// The prefix to use to hide lines visually but still execute them.
+    pub hidden_line_prefix: Option<String>,
 }
 
 #[derive(Clone, Debug, Default, Deserialize, ValueEnum, JsonSchema)]
@@ -330,6 +345,10 @@ pub struct KeyBindingsConfig {
     /// The key binding to close the application.
     #[serde(default = "default_exit_bindings")]
     pub(crate) exit: Vec<KeyBinding>,
+
+    /// The key binding to suspend the application.
+    #[serde(default = "default_suspend_bindings")]
+    pub(crate) suspend: Vec<KeyBinding>,
 }
 
 impl Default for KeyBindingsConfig {
@@ -348,6 +367,7 @@ impl Default for KeyBindingsConfig {
             toggle_bindings: default_toggle_bindings_modal_bindings(),
             close_modal: default_close_modal_bindings(),
             exit: default_exit_bindings(),
+            suspend: default_suspend_bindings(),
         }
     }
 }
@@ -409,7 +429,11 @@ fn default_close_modal_bindings() -> Vec<KeyBinding> {
 }
 
 fn default_exit_bindings() -> Vec<KeyBinding> {
-    make_keybindings(["<c-c>"])
+    make_keybindings(["<c-c>", "q"])
+}
+
+fn default_suspend_bindings() -> Vec<KeyBinding> {
+    make_keybindings(["<c-z>"])
 }
 
 #[cfg(test)]
