@@ -114,9 +114,8 @@ where
 mod test {
     use super::*;
     use crate::{
-        presentation::{
-            AsRenderOperations, BlockLine, BlockLineText, RenderAsync, RenderAsyncState, Slide, SlideBuilder,
-        },
+        markdown::text::WeightedTextBlock,
+        presentation::{AsRenderOperations, BlockLine, RenderAsync, RenderAsyncState, Slide, SlideBuilder},
         render::properties::WindowSize,
         style::{Color, Colors},
         theme::{Alignment, Margin},
@@ -131,10 +130,6 @@ mod test {
         fn as_render_operations(&self, _dimensions: &WindowSize) -> Vec<RenderOperation> {
             Vec::new()
         }
-
-        fn diffable_content(&self) -> Option<&str> {
-            None
-        }
     }
 
     impl RenderAsync for Dynamic {
@@ -156,10 +151,13 @@ mod test {
     #[case(RenderOperation::RenderText{line: String::from("asd").into(), alignment: Default::default()})]
     #[case(RenderOperation::RenderBlockLine(
         BlockLine{
-            text: BlockLineText::Preformatted("".into()),
+            prefix: "".into(),
+            right_padding_length: 0,
+            repeat_prefix_on_wrap: false,
+            text: WeightedTextBlock::from("".to_string()),
             alignment: Default::default(),
             block_length: 42,
-            unformatted_length: 1337
+            block_color: None,
         }
     ))]
     #[case(RenderOperation::RenderDynamic(Rc::new(Dynamic)))]
