@@ -2,7 +2,7 @@
 
 use crate::{
     custom::LanguageSnippetExecutionConfig,
-    markdown::elements::{Snippet, SnippetLanguage},
+    processing::code::{Snippet, SnippetLanguage},
 };
 use once_cell::sync::Lazy;
 use os_pipe::PipeReader;
@@ -10,6 +10,7 @@ use std::{
     collections::{BTreeMap, HashMap},
     fs::File,
     io::{self, BufRead, BufReader, Write},
+    path::{Path, PathBuf},
     process::{self, Child, Stdio},
     sync::{Arc, Mutex},
     thread,
@@ -23,11 +24,13 @@ static EXECUTORS: Lazy<BTreeMap<SnippetLanguage, LanguageSnippetExecutionConfig>
 #[derive(Debug)]
 pub struct SnippetExecutor {
     executors: BTreeMap<SnippetLanguage, LanguageSnippetExecutionConfig>,
+    cwd: PathBuf,
 }
 
 impl SnippetExecutor {
     pub fn new(
         custom_executors: BTreeMap<SnippetLanguage, LanguageSnippetExecutionConfig>,
+        cwd: PathBuf,
     ) -> Result<Self, InvalidSnippetConfig> {
         let mut executors = EXECUTORS.clone();
         executors.extend(custom_executors);
@@ -44,44 +47,82 @@ impl SnippetExecutor {
                 }
             }
         }
-        Ok(Self { executors })
+        Ok(Self { executors, cwd })
     }
 
     pub(crate) fn is_execution_supported(&self, language: &SnippetLanguage) -> bool {
         self.executors.contains_key(language)
     }
 
-    /// Execute a piece of code.
-    pub(crate) fn execute(&self, code: &Snippet) -> Result<ExecutionHandle, CodeExecuteError> {
-        if !code.attributes.execute {
-            return Err(CodeExecuteError::NotExecutableCode);
-        }
-        let Some(config) = self.executors.get(&code.language) else {
-            return Err(CodeExecuteError::UnsupportedExecution);
-        };
-        Self::execute_lang(config, code.executable_contents().as_bytes())
+    /// Execute a piece of code asynchronously.
+    pub(crate) fn execute_async(&self, snippet: &Snippet) -> Result<ExecutionHandle, CodeExecuteError> {
+        let config = self.language_config(snippet)?;
+        let script_dir = Self::write_snippet(snippet, config)?;
+        let state: Arc<Mutex<ExecutionState>> = Default::default();
+        let reader_handle = CommandsRunner::spawn(
+            state.clone(),
+            script_dir,
+            config.commands.clone(),
+            config.environment.clone(),
+            self.cwd.to_path_buf(),
+        );
+        let handle = ExecutionHandle { state, reader_handle };
+        Ok(handle)
     }
 
-    fn execute_lang(config: &LanguageSnippetExecutionConfig, code: &[u8]) -> Result<ExecutionHandle, CodeExecuteError> {
+    /// Executes a piece of code synchronously.
+    pub(crate) fn execute_sync(&self, snippet: &Snippet) -> Result<(), CodeExecuteError> {
+        let config = self.language_config(snippet)?;
+        let script_dir = Self::write_snippet(snippet, config)?;
+        let script_dir_path = script_dir.path().to_string_lossy();
+        for mut commands in config.commands.clone() {
+            for command in &mut commands {
+                *command = command.replace("$pwd", &script_dir_path);
+            }
+            let (command, args) = commands.split_first().expect("no commands");
+            let child = process::Command::new(command)
+                .args(args)
+                .envs(&config.environment)
+                .current_dir(&self.cwd)
+                .stderr(Stdio::piped())
+                .spawn()
+                .map_err(|e| CodeExecuteError::SpawnProcess(command.clone(), e))?;
+
+            let output = child.wait_with_output().map_err(CodeExecuteError::Waiting)?;
+            if !output.status.success() {
+                let error = String::from_utf8_lossy(&output.stderr).to_string();
+                return Err(CodeExecuteError::Running(error));
+            }
+        }
+        Ok(())
+    }
+
+    pub(crate) fn hidden_line_prefix(&self, language: &SnippetLanguage) -> Option<&str> {
+        self.executors.get(language).and_then(|lang| lang.hidden_line_prefix.as_deref())
+    }
+
+    fn language_config(&self, snippet: &Snippet) -> Result<&LanguageSnippetExecutionConfig, CodeExecuteError> {
+        if !snippet.attributes.execute && !snippet.attributes.execute_replace {
+            return Err(CodeExecuteError::NotExecutableCode);
+        }
+        self.executors.get(&snippet.language).ok_or(CodeExecuteError::UnsupportedExecution)
+    }
+
+    fn write_snippet(snippet: &Snippet, config: &LanguageSnippetExecutionConfig) -> Result<TempDir, CodeExecuteError> {
+        let hide_prefix = config.hidden_line_prefix.as_deref();
+        let code = snippet.executable_contents(hide_prefix);
         let script_dir =
             tempfile::Builder::default().prefix(".presenterm").tempdir().map_err(CodeExecuteError::TempDir)?;
         let snippet_path = script_dir.path().join(&config.filename);
-        {
-            let mut snippet_file = File::create(snippet_path).map_err(CodeExecuteError::TempDir)?;
-            snippet_file.write_all(code).map_err(CodeExecuteError::TempDir)?;
-        }
-
-        let state: Arc<Mutex<ExecutionState>> = Default::default();
-        let reader_handle =
-            CommandsRunner::spawn(state.clone(), script_dir, config.commands.clone(), config.environment.clone());
-        let handle = ExecutionHandle { state, reader_handle };
-        Ok(handle)
+        let mut snippet_file = File::create(snippet_path).map_err(CodeExecuteError::TempDir)?;
+        snippet_file.write_all(code.as_bytes()).map_err(CodeExecuteError::TempDir)?;
+        Ok(script_dir)
     }
 }
 
 impl Default for SnippetExecutor {
     fn default() -> Self {
-        Self::new(Default::default()).expect("initialization failed")
+        Self::new(Default::default(), PathBuf::from("./")).expect("initialization failed")
     }
 }
 
@@ -107,6 +148,12 @@ pub(crate) enum CodeExecuteError {
 
     #[error("error creating pipe: {0}")]
     Pipe(io::Error),
+
+    #[error("error waiting for process to run: {0}")]
+    Waiting(io::Error),
+
+    #[error("error running process: {0}")]
+    Running(String),
 }
 
 /// A handle for the execution of a piece of code.
@@ -129,15 +176,16 @@ impl CommandsRunner {
         script_directory: TempDir,
         commands: Vec<Vec<String>>,
         env: HashMap<String, String>,
+        cwd: PathBuf,
     ) -> thread::JoinHandle<()> {
         let reader = Self { state, script_directory };
-        thread::spawn(|| reader.run(commands, env))
+        thread::spawn(|| reader.run(commands, env, cwd))
     }
 
-    fn run(self, commands: Vec<Vec<String>>, env: HashMap<String, String>) {
+    fn run(self, commands: Vec<Vec<String>>, env: HashMap<String, String>, cwd: PathBuf) {
         let mut last_result = true;
         for command in commands {
-            last_result = self.run_command(command, &env);
+            last_result = self.run_command(command, &env, &cwd);
             if !last_result {
                 break;
             }
@@ -149,8 +197,8 @@ impl CommandsRunner {
         self.state.lock().unwrap().status = status;
     }
 
-    fn run_command(&self, command: Vec<String>, env: &HashMap<String, String>) -> bool {
-        let (mut child, reader) = match self.launch_process(command, env) {
+    fn run_command(&self, command: Vec<String>, env: &HashMap<String, String>, cwd: &Path) -> bool {
+        let (mut child, reader) = match self.launch_process(command, env, cwd) {
             Ok(inner) => inner,
             Err(e) => {
                 let mut state = self.state.lock().unwrap();
@@ -171,17 +219,19 @@ impl CommandsRunner {
         &self,
         mut commands: Vec<String>,
         env: &HashMap<String, String>,
+        cwd: &Path,
     ) -> Result<(Child, PipeReader), CodeExecuteError> {
         let (reader, writer) = os_pipe::pipe().map_err(CodeExecuteError::Pipe)?;
         let writer_clone = writer.try_clone().map_err(CodeExecuteError::Pipe)?;
+        let script_dir = self.script_directory.path().to_string_lossy();
         for command in &mut commands {
-            *command = command.replace("$pwd", &self.script_directory.path().to_string_lossy());
+            *command = command.replace("$pwd", &script_dir);
         }
         let (command, args) = commands.split_first().expect("no commands");
         let child = process::Command::new(command)
             .args(args)
             .envs(env)
-            .current_dir(self.script_directory.path())
+            .current_dir(cwd)
             .stdin(Stdio::null())
             .stdout(writer)
             .stderr(writer_clone)
@@ -230,7 +280,7 @@ impl ProcessStatus {
 #[cfg(test)]
 mod test {
     use super::*;
-    use crate::markdown::elements::SnippetAttributes;
+    use crate::processing::code::SnippetAttributes;
 
     #[test]
     fn shell_code_execution() {
@@ -243,7 +293,7 @@ echo 'bye'"
             language: SnippetLanguage::Shell,
             attributes: SnippetAttributes { execute: true, ..Default::default() },
         };
-        let handle = SnippetExecutor::default().execute(&code).expect("execution failed");
+        let handle = SnippetExecutor::default().execute_async(&code).expect("execution failed");
         let state = loop {
             let state = handle.state.lock().unwrap();
             if state.status.is_finished() {
@@ -263,7 +313,7 @@ echo 'bye'"
             language: SnippetLanguage::Shell,
             attributes: SnippetAttributes { execute: false, ..Default::default() },
         };
-        let result = SnippetExecutor::default().execute(&code);
+        let result = SnippetExecutor::default().execute_async(&code);
         assert!(result.is_err());
     }
 
@@ -279,7 +329,7 @@ echo 'hello world'
             language: SnippetLanguage::Shell,
             attributes: SnippetAttributes { execute: true, ..Default::default() },
         };
-        let handle = SnippetExecutor::default().execute(&code).expect("execution failed");
+        let handle = SnippetExecutor::default().execute_async(&code).expect("execution failed");
         let state = loop {
             let state = handle.state.lock().unwrap();
             if state.status.is_finished() {
@@ -304,7 +354,7 @@ echo 'hello world'
             language: SnippetLanguage::Shell,
             attributes: SnippetAttributes { execute: true, ..Default::default() },
         };
-        let handle = SnippetExecutor::default().execute(&code).expect("execution failed");
+        let handle = SnippetExecutor::default().execute_async(&code).expect("execution failed");
         let state = loop {
             let state = handle.state.lock().unwrap();
             if state.status.is_finished() {
@@ -319,6 +369,6 @@ echo 'hello world'
 
     #[test]
     fn built_in_executors() {
-        SnippetExecutor::new(Default::default()).expect("invalid default executors");
+        SnippetExecutor::new(Default::default(), PathBuf::from("./")).expect("invalid default executors");
     }
 }
