@@ -1,4 +1,4 @@
-use clap::{error::ErrorKind, CommandFactory, Parser};
+use clap::{CommandFactory, Parser, error::ErrorKind};
 use comrak::Arena;
 use directories::ProjectDirs;
 use presenterm::{
@@ -8,7 +8,8 @@ use presenterm::{
     ValidateOverflows,
 };
 use std::{
-    env, io,
+    env::{self, current_dir},
+    io,
     path::{Path, PathBuf},
     rc::Rc,
     sync::Arc,
@@ -69,6 +70,10 @@ struct Cli {
     #[clap(short = 'x', long)]
     enable_snippet_execution: bool,
 
+    /// Enable code snippet auto execution via `+exec_replace` blocks.
+    #[clap(short = 'X', long)]
+    enable_snippet_execution_replace: bool,
+
     /// The path to the configuration file.
     #[clap(short, long)]
     config_file: Option<String>,
@@ -95,7 +100,10 @@ struct Customizations {
     code_executor: SnippetExecutor,
 }
 
-fn load_customizations(config_file_path: Option<PathBuf>) -> Result<Customizations, Box<dyn std::error::Error>> {
+fn load_customizations(
+    config_file_path: Option<PathBuf>,
+    cwd: &Path,
+) -> Result<Customizations, Box<dyn std::error::Error>> {
     let configs_path: PathBuf = match env::var("XDG_CONFIG_HOME") {
         Ok(path) => Path::new(&path).join("presenterm"),
         Err(_) => {
@@ -108,7 +116,7 @@ fn load_customizations(config_file_path: Option<PathBuf>) -> Result<Customizatio
     let themes = load_themes(&configs_path)?;
     let config_file_path = config_file_path.unwrap_or_else(|| configs_path.join("config.yaml"));
     let config = Config::load(&config_file_path)?;
-    let code_executor = SnippetExecutor::new(config.snippet.exec.custom.clone())?;
+    let code_executor = SnippetExecutor::new(config.snippet.exec.custom.clone(), cwd.to_path_buf())?;
     Ok(Customizations { config, themes, code_executor })
 }
 
@@ -142,6 +150,7 @@ fn make_builder_options(config: &Config, mode: &PresentMode, force_default_theme
         print_modal_background: false,
         strict_front_matter_parsing: config.options.strict_front_matter_parsing.unwrap_or(true),
         enable_snippet_execution: config.snippet.exec.enable,
+        enable_snippet_execution_replace: config.snippet.exec_replace.enable,
     }
 }
 
@@ -185,10 +194,29 @@ fn run(mut cli: Cli) -> Result<(), Box<dyn std::error::Error>> {
         let schema = schemars::schema_for!(Config);
         serde_json::to_writer_pretty(io::stdout(), &schema).map_err(|e| format!("failed to write schema: {e}"))?;
         return Ok(());
+    } else if cli.acknowledgements {
+        display_acknowledgements();
+        return Ok(());
+    } else if cli.list_themes {
+        let Customizations { config, themes, .. } =
+            load_customizations(cli.config_file.clone().map(PathBuf::from), &current_dir()?)?;
+        let bindings = config.bindings.try_into()?;
+        let demo = ThemesDemo::new(themes, bindings, io::stdout())?;
+        demo.run()?;
+        return Ok(());
     }
 
+    let Some(path) = cli.path.take() else {
+        Cli::command().error(ErrorKind::MissingRequiredArgument, "no path specified").exit();
+    };
+    let mut resources_path = path.parent().unwrap_or(Path::new("./")).to_path_buf();
+    if resources_path == Path::new("") {
+        resources_path = "./".into();
+    }
+    let resources_path = resources_path.canonicalize().unwrap_or(resources_path);
+
     let Customizations { config, themes, code_executor } =
-        load_customizations(cli.config_file.clone().map(PathBuf::from))?;
+        load_customizations(cli.config_file.clone().map(PathBuf::from), &resources_path)?;
 
     let default_theme = load_default_theme(&config, &themes, &cli);
     let force_default_theme = cli.theme.is_some();
@@ -199,35 +227,25 @@ fn run(mut cli: Cli) -> Result<(), Box<dyn std::error::Error>> {
     };
     let arena = Arena::new();
     let parser = MarkdownParser::new(&arena);
-    if cli.acknowledgements {
-        display_acknowledgements();
-        return Ok(());
-    } else if cli.list_themes {
-        let bindings = config.bindings.try_into()?;
-        let demo = ThemesDemo::new(themes, bindings, io::stdout())?;
-        demo.run()?;
-        return Ok(());
-    }
 
-    let path = cli.path.take().unwrap_or_else(|| {
-        Cli::command().error(ErrorKind::MissingRequiredArgument, "no path specified").exit();
-    });
     let validate_overflows = overflow_validation(&mode, &config.defaults.validate_overflows) || cli.validate_overflows;
-    let resources_path = path.parent().unwrap_or(Path::new("/"));
     let mut options = make_builder_options(&config, &mode, force_default_theme);
     if cli.enable_snippet_execution {
         options.enable_snippet_execution = true;
     }
+    if cli.enable_snippet_execution_replace {
+        options.enable_snippet_execution_replace = true;
+    }
     let graphics_mode = select_graphics_mode(&cli, &config);
     let printer = Arc::new(ImagePrinter::new(graphics_mode.clone())?);
     let registry = ImageRegistry(printer.clone());
-    let resources = Resources::new(resources_path, registry.clone());
+    let resources = Resources::new(resources_path.clone(), registry.clone());
     let third_party_config = ThirdPartyConfigs {
         typst_ppi: config.typst.ppi.to_string(),
         mermaid_scale: config.mermaid.scale.to_string(),
         threads: config.snippet.render.threads,
     };
-    let third_party = ThirdPartyRender::new(third_party_config, registry, resources_path);
+    let third_party = ThirdPartyRender::new(third_party_config, registry, &resources_path);
     let code_executor = Rc::new(code_executor);
     if cli.export_pdf || cli.generate_pdf_metadata {
         let mut exporter =
@@ -242,6 +260,9 @@ fn run(mut cli: Cli) -> Result<(), Box<dyn std::error::Error>> {
         if cli.enable_snippet_execution {
             args.push("-x");
         }
+        if cli.enable_snippet_execution_replace {
+            args.push("-X");
+        }
         if cli.export_pdf {
             exporter.export_pdf(&path, &args)?;
         } else {
@@ -249,7 +270,7 @@ fn run(mut cli: Cli) -> Result<(), Box<dyn std::error::Error>> {
             println!("{}", serde_json::to_string_pretty(&meta)?);
         }
     } else {
-        let commands = CommandSource::new(&path, config.bindings.clone())?;
+        let commands = CommandSource::new(config.bindings.clone())?;
         options.print_modal_background = matches!(graphics_mode, GraphicsMode::Kitty { .. });
 
         let options = PresenterOptions {
