@@ -1,24 +1,21 @@
-use super::{code::CodeBlockParseError, elements::SourcePosition};
+use super::elements::SourcePosition;
 use crate::{
-    markdown::{
-        code::CodeBlockParser,
-        elements::{ListItem, ListItemType, MarkdownElement, ParagraphElement, Table, TableRow, Text, TextBlock},
-    },
+    markdown::elements::{ListItem, ListItemType, MarkdownElement, Table, TableRow, Text, TextBlock},
     style::TextStyle,
 };
 use comrak::{
+    Arena, ComrakOptions,
     arena_tree::Node,
     format_commonmark,
     nodes::{
         Ast, AstNode, ListDelimType, ListType, NodeCodeBlock, NodeHeading, NodeHtmlBlock, NodeList, NodeValue,
         Sourcepos,
     },
-    parse_document, Arena, ComrakOptions, ListStyleType,
+    parse_document,
 };
 use std::{
     cell::RefCell,
     fmt::{self, Debug, Display},
-    io::BufWriter,
     mem,
 };
 
@@ -77,13 +74,13 @@ impl<'a> MarkdownParser<'a> {
                 | MarkdownElement::SetexHeading { .. }
                 | MarkdownElement::Heading { .. }
                 | MarkdownElement::Paragraph(_)
-                | MarkdownElement::Image { .. }
                 | MarkdownElement::List(_)
-                | MarkdownElement::Snippet(_)
                 | MarkdownElement::Table(_)
                 | MarkdownElement::ThematicBreak
                 | MarkdownElement::BlockQuote(_) => continue,
-                MarkdownElement::Comment { source_position, .. } => source_position,
+                MarkdownElement::Comment { source_position, .. }
+                | MarkdownElement::Snippet { source_position, .. }
+                | MarkdownElement::Image { source_position, .. } => source_position,
             };
             *position = position.offset_lines(lines_offset);
         }
@@ -104,7 +101,7 @@ impl<'a> MarkdownParser<'a> {
             NodeValue::CodeBlock(block) => Self::parse_code_block(block, data.sourcepos)?,
             NodeValue::ThematicBreak => MarkdownElement::ThematicBreak,
             NodeValue::HtmlBlock(block) => Self::parse_html_block(block, data.sourcepos)?,
-            NodeValue::BlockQuote | NodeValue::MultilineBlockQuote(_) => Self::parse_block_quote(node)?,
+            NodeValue::BlockQuote | NodeValue::MultilineBlockQuote(_) => self.parse_block_quote(node)?,
             other => return Err(ParseErrorKind::UnsupportedElement(other.identifier()).with_sourcepos(data.sourcepos)),
         };
         Ok(vec![element])
@@ -134,42 +131,31 @@ impl<'a> MarkdownParser<'a> {
         Ok(MarkdownElement::Comment { comment: block.into(), source_position: sourcepos.into() })
     }
 
-    fn parse_block_quote(node: &'a AstNode<'a>) -> ParseResult<MarkdownElement> {
-        // This renders the contents of this block quote AST as commonmark, given we otherwise
-        // would need to either do this outselves or pull the raw block contents off of the
-        // original raw string and that also isn't great.
-        let mut buffer = BufWriter::new(Vec::new());
-        let mut options = ParserOptions::default().0;
-        options.render.list_style = ListStyleType::Star;
-        format_commonmark(node, &options, &mut buffer)
-            .map_err(|e| ParseErrorKind::Internal(e.to_string()).with_sourcepos(node.data.borrow().sourcepos))?;
-
-        let buffer = buffer.into_inner().expect("unwrapping writer failed");
-        let mut lines = Vec::new();
-        for line in String::from_utf8_lossy(&buffer).lines() {
-            let line = match line.find('>') {
-                Some(index) => line[index + 1..].trim(),
-                None => line,
-            };
-            let mut line = line.to_string();
-            // `format_commonmark` escapes these symbols so we un-escape them.
-            for escape in &["\\*", "\\!", "\\[", "\\]", "\\#", "\\`", "\\<", "\\>"] {
-                if line.contains(escape) {
-                    line = line.replace(escape, &escape[1..]);
-                }
+    fn parse_block_quote(&self, node: &'a AstNode<'a>) -> ParseResult<MarkdownElement> {
+        let mut elements = Vec::new();
+        let inlines = InlinesParser::new(self.arena, SoftBreak::Newline, StringifyImages::Yes).parse(node)?;
+        for inline in inlines {
+            match inline {
+                Inline::Text(text) => elements.push(text),
+                Inline::LineBreak => elements.push(TextBlock::from("")),
+                Inline::Image { .. } => {}
             }
-            lines.push(line);
         }
-        Ok(MarkdownElement::BlockQuote(lines))
+        if elements.last() == Some(&TextBlock::from("")) {
+            elements.pop();
+        }
+        Ok(MarkdownElement::BlockQuote(elements))
     }
 
     fn parse_code_block(block: &NodeCodeBlock, sourcepos: Sourcepos) -> ParseResult<MarkdownElement> {
         if !block.fenced {
             return Err(ParseErrorKind::UnfencedCodeBlock.with_sourcepos(sourcepos));
         }
-        let code =
-            CodeBlockParser::parse(block).map_err(|e| ParseErrorKind::InvalidCodeBlock(e).with_sourcepos(sourcepos))?;
-        Ok(MarkdownElement::Snippet(code))
+        Ok(MarkdownElement::Snippet {
+            info: block.info.clone(),
+            code: block.literal.clone(),
+            source_position: sourcepos.into(),
+        })
     }
 
     fn parse_heading(&self, heading: &NodeHeading, node: &'a AstNode<'a>) -> ParseResult<MarkdownElement> {
@@ -183,12 +169,12 @@ impl<'a> MarkdownParser<'a> {
 
     fn parse_paragraph(&self, node: &'a AstNode<'a>) -> ParseResult<Vec<MarkdownElement>> {
         let mut elements = Vec::new();
-        let inlines = InlinesParser::new(self.arena).parse(node)?;
+        let inlines = InlinesParser::new(self.arena, SoftBreak::Space, StringifyImages::No).parse(node)?;
         let mut paragraph_elements = Vec::new();
         for inline in inlines {
             match inline {
-                Inline::Text(text) => paragraph_elements.push(ParagraphElement::Text(text)),
-                Inline::LineBreak => paragraph_elements.push(ParagraphElement::LineBreak),
+                Inline::Text(text) => paragraph_elements.push(text),
+                Inline::LineBreak => (),
                 Inline::Image { path, title } => {
                     if !paragraph_elements.is_empty() {
                         elements.push(MarkdownElement::Paragraph(mem::take(&mut paragraph_elements)));
@@ -208,7 +194,7 @@ impl<'a> MarkdownParser<'a> {
     }
 
     fn parse_text(&self, node: &'a AstNode<'a>) -> ParseResult<TextBlock> {
-        let inlines = InlinesParser::new(self.arena).parse(node)?;
+        let inlines = InlinesParser::new(self.arena, SoftBreak::Space, StringifyImages::No).parse(node)?;
         let mut chunks = Vec::new();
         for inline in inlines {
             match inline {
@@ -311,15 +297,27 @@ impl<'a> MarkdownParser<'a> {
     }
 }
 
+enum SoftBreak {
+    Newline,
+    Space,
+}
+
+enum StringifyImages {
+    Yes,
+    No,
+}
+
 struct InlinesParser<'a> {
     inlines: Vec<Inline>,
     pending_text: Vec<Text>,
     arena: &'a Arena<AstNode<'a>>,
+    soft_break: SoftBreak,
+    stringify_images: StringifyImages,
 }
 
 impl<'a> InlinesParser<'a> {
-    fn new(arena: &'a Arena<AstNode<'a>>) -> Self {
-        Self { inlines: Vec::new(), pending_text: Vec::new(), arena }
+    fn new(arena: &'a Arena<AstNode<'a>>, soft_break: SoftBreak, stringify_images: StringifyImages) -> Self {
+        Self { inlines: Vec::new(), pending_text: Vec::new(), arena, soft_break, stringify_images }
     }
 
     fn parse(mut self, node: &'a AstNode<'a>) -> ParseResult<Vec<Inline>> {
@@ -335,25 +333,51 @@ impl<'a> InlinesParser<'a> {
         }
     }
 
-    fn process_node(&mut self, node: &'a AstNode<'a>, style: TextStyle) -> ParseResult<()> {
+    fn process_node(&mut self, node: &'a AstNode<'a>, parent: &'a AstNode<'a>, style: TextStyle) -> ParseResult<()> {
         let data = node.data.borrow();
         match &data.value {
             NodeValue::Text(text) => {
-                self.pending_text.push(Text::new(text.clone(), style.clone()));
+                self.pending_text.push(Text::new(text.clone(), style));
             }
             NodeValue::Code(code) => {
                 self.pending_text.push(Text::new(code.literal.clone(), TextStyle::default().code()));
             }
-            NodeValue::Strong => self.process_children(node, style.clone().bold())?,
-            NodeValue::Emph => self.process_children(node, style.clone().italics())?,
-            NodeValue::Strikethrough => self.process_children(node, style.clone().strikethrough())?,
-            NodeValue::SoftBreak => self.pending_text.push(Text::from(" ")),
-            NodeValue::Link(link) => self.pending_text.push(Text::new(link.url.clone(), TextStyle::default().link())),
+            NodeValue::Strong => self.process_children(node, style.bold())?,
+            NodeValue::Emph => self.process_children(node, style.italics())?,
+            NodeValue::Strikethrough => self.process_children(node, style.strikethrough())?,
+            NodeValue::SoftBreak => {
+                match self.soft_break {
+                    SoftBreak::Newline => {
+                        self.store_pending_text();
+                    }
+                    SoftBreak::Space => self.pending_text.push(Text::from(" ")),
+                };
+            }
+            NodeValue::Link(link) => {
+                let has_label = node.first_child().is_some();
+                if has_label {
+                    self.process_children(node, TextStyle::default().link_label())?;
+                    self.pending_text.push(Text::from(" ("));
+                }
+                self.pending_text.push(Text::new(link.url.clone(), TextStyle::default().link_url()));
+                if !link.title.is_empty() {
+                    self.pending_text.push(Text::from(" \""));
+                    self.pending_text.push(Text::new(link.title.clone(), TextStyle::default().link_title()));
+                    self.pending_text.push(Text::from("\""));
+                }
+                if has_label {
+                    self.pending_text.push(Text::from(")"));
+                }
+            }
             NodeValue::LineBreak => {
                 self.store_pending_text();
                 self.inlines.push(Inline::LineBreak);
             }
             NodeValue::Image(link) => {
+                if matches!(self.stringify_images, StringifyImages::Yes) {
+                    self.pending_text.push(Text::from(format!("![{}]({})", link.title, link.url)));
+                    return Ok(());
+                }
                 self.store_pending_text();
 
                 // The image "title" contains inlines so we create a dummy paragraph node that
@@ -371,6 +395,30 @@ impl<'a> InlinesParser<'a> {
                 let title = String::from_utf8_lossy(&buffer).trim_end().to_string();
                 self.inlines.push(Inline::Image { path: link.url.clone(), title });
             }
+            NodeValue::Paragraph => {
+                self.process_children(node, style)?;
+                self.store_pending_text();
+                if matches!(parent.data.borrow().value, NodeValue::BlockQuote | NodeValue::MultilineBlockQuote(_)) {
+                    self.inlines.push(Inline::LineBreak);
+                }
+            }
+            NodeValue::List(_) => {
+                self.process_children(node, style)?;
+                self.store_pending_text();
+                self.inlines.push(Inline::LineBreak);
+            }
+            NodeValue::Item(item) => {
+                match (item.list_type, item.delimiter) {
+                    (ListType::Bullet, _) => self.pending_text.push(Text::from("* ")),
+                    (ListType::Ordered, ListDelimType::Period) => {
+                        self.pending_text.push(Text::from(format!("{}. ", item.start)))
+                    }
+                    (ListType::Ordered, ListDelimType::Paren) => {
+                        self.pending_text.push(Text::from(format!("{}) ", item.start)))
+                    }
+                };
+                self.process_children(node, style)?;
+            }
             other => {
                 return Err(ParseErrorKind::UnsupportedStructure { container: "text", element: other.identifier() }
                     .with_sourcepos(data.sourcepos));
@@ -379,9 +427,9 @@ impl<'a> InlinesParser<'a> {
         Ok(())
     }
 
-    fn process_children(&mut self, node: &'a AstNode<'a>, style: TextStyle) -> ParseResult<()> {
-        for node in node.children() {
-            self.process_node(node, style.clone())?;
+    fn process_children(&mut self, root: &'a AstNode<'a>, style: TextStyle) -> ParseResult<()> {
+        for node in root.children() {
+            self.process_node(node, root, style)?;
         }
         Ok(())
     }
@@ -437,9 +485,6 @@ pub(crate) enum ParseErrorKind {
     /// We don't support unfenced code blocks.
     UnfencedCodeBlock,
 
-    /// A code block contains invalid attributes.
-    InvalidCodeBlock(CodeBlockParseError),
-
     /// An internal parsing error.
     Internal(String),
 }
@@ -452,7 +497,6 @@ impl Display for ParseErrorKind {
                 write!(f, "unsupported structure in {container}: {element}")
             }
             Self::UnfencedCodeBlock => write!(f, "only fenced code blocks are supported"),
-            Self::InvalidCodeBlock(error) => write!(f, "invalid code block: {error}"),
             Self::Internal(message) => write!(f, "internal error: {message}"),
         }
     }
@@ -516,7 +560,6 @@ impl Identifier for NodeValue {
 #[cfg(test)]
 mod test {
     use super::*;
-    use crate::markdown::elements::SnippetLanguage;
     use rstest::rstest;
     use std::path::Path;
 
@@ -566,17 +609,69 @@ boop
             Text::new("strikethrough", TextStyle::default().strikethrough()),
         ];
 
-        let expected_elements = &[ParagraphElement::Text(TextBlock(expected_chunks))];
+        let expected_elements = &[TextBlock(expected_chunks)];
         assert_eq!(elements, expected_elements);
     }
 
     #[test]
-    fn link() {
+    fn link_wo_label_wo_title() {
+        let parsed = parse_single("my [](https://example.com)");
+        let MarkdownElement::Paragraph(elements) = parsed else { panic!("not a paragraph: {parsed:?}") };
+        let expected_chunks =
+            vec![Text::from("my "), Text::new("https://example.com", TextStyle::default().link_url())];
+
+        let expected_elements = &[TextBlock(expected_chunks)];
+        assert_eq!(elements, expected_elements);
+    }
+
+    #[test]
+    fn link_w_label_wo_title() {
         let parsed = parse_single("my [website](https://example.com)");
         let MarkdownElement::Paragraph(elements) = parsed else { panic!("not a paragraph: {parsed:?}") };
-        let expected_chunks = vec![Text::from("my "), Text::new("https://example.com", TextStyle::default().link())];
+        let expected_chunks = vec![
+            Text::from("my "),
+            Text::new("website", TextStyle::default().link_label()),
+            Text::from(" ("),
+            Text::new("https://example.com", TextStyle::default().link_url()),
+            Text::from(")"),
+        ];
 
-        let expected_elements = &[ParagraphElement::Text(TextBlock(expected_chunks))];
+        let expected_elements = &[TextBlock(expected_chunks)];
+        assert_eq!(elements, expected_elements);
+    }
+
+    #[test]
+    fn link_wo_label_w_title() {
+        let parsed = parse_single("my [](https://example.com \"Example\")");
+        let MarkdownElement::Paragraph(elements) = parsed else { panic!("not a paragraph: {parsed:?}") };
+        let expected_chunks = vec![
+            Text::from("my "),
+            Text::new("https://example.com", TextStyle::default().link_url()),
+            Text::from(" \""),
+            Text::new("Example", TextStyle::default().link_title()),
+            Text::from("\""),
+        ];
+
+        let expected_elements = &[TextBlock(expected_chunks)];
+        assert_eq!(elements, expected_elements);
+    }
+
+    #[test]
+    fn link_w_label_w_title() {
+        let parsed = parse_single("my [website](https://example.com \"Example\")");
+        let MarkdownElement::Paragraph(elements) = parsed else { panic!("not a paragraph: {parsed:?}") };
+        let expected_chunks = vec![
+            Text::from("my "),
+            Text::new("website", TextStyle::default().link_label()),
+            Text::from(" ("),
+            Text::new("https://example.com", TextStyle::default().link_url()),
+            Text::from(" \""),
+            Text::new("Example", TextStyle::default().link_title()),
+            Text::from("\""),
+            Text::from(")"),
+        ];
+
+        let expected_elements = &[TextBlock(expected_chunks)];
         assert_eq!(elements, expected_elements);
     }
 
@@ -654,42 +749,25 @@ another",
         assert_eq!(parsed.len(), 2);
 
         let MarkdownElement::Paragraph(elements) = &parsed[0] else { panic!("not a line break: {parsed:?}") };
-        assert_eq!(elements.len(), 3);
+        assert_eq!(elements.len(), 2);
 
         let expected_chunks = &[Text::from("some text"), Text::from(" "), Text::from("with line breaks")];
-        let ParagraphElement::Text(text) = &elements[0] else { panic!("non-text in paragraph") };
+        let text = &elements[0];
         assert_eq!(text.0, expected_chunks);
-        assert!(matches!(&elements[1], ParagraphElement::LineBreak));
-        assert!(matches!(&elements[2], ParagraphElement::Text(_)));
     }
 
     #[test]
     fn code_block() {
         let parsed = parse_single(
             r"
-```rust
+```rust +exec
 let q = 42;
 ````
 ",
         );
-        let MarkdownElement::Snippet(code) = parsed else { panic!("not a code block: {parsed:?}") };
-        assert_eq!(code.language, SnippetLanguage::Rust);
-        assert_eq!(code.contents, "let q = 42;\n");
-        assert!(!code.attributes.execute);
-    }
-
-    #[test]
-    fn executable_code_block() {
-        let parsed = parse_single(
-            r"
-```bash +exec
-echo hi mom
-````
-",
-        );
-        let MarkdownElement::Snippet(code) = parsed else { panic!("not a code block: {parsed:?}") };
-        assert_eq!(code.language, SnippetLanguage::Bash);
-        assert!(code.attributes.execute);
+        let MarkdownElement::Snippet { info, code, .. } = parsed else { panic!("not a code block: {parsed:?}") };
+        assert_eq!(info, "rust +exec");
+        assert_eq!(code, "let q = 42;\n");
     }
 
     #[test]
@@ -699,7 +777,7 @@ echo hi mom
         let expected_chunks = &[Text::from("some "), Text::new("inline code", TextStyle::default().code())];
         assert_eq!(elements.len(), 1);
 
-        let ParagraphElement::Text(text) = &elements[0] else { panic!("non-text in paragraph") };
+        let text = &elements[0];
         assert_eq!(text.0, expected_chunks);
     }
 
@@ -749,20 +827,38 @@ echo hi mom
     fn block_quote() {
         let parsed = parse_single(
             r#"
-> bar!@#$%^&*()[]'"{}-=`~,.<>/?
-> foo
+> foo **is not** bar
+> ![](hehe.png) test ![](potato.png)
 > 
 > * a
 > * b
+>
+> 1. a
+> 2. b
+>
+> 1) a
+> 2) b
 "#,
         );
         let MarkdownElement::BlockQuote(lines) = parsed else { panic!("not a block quote: {parsed:?}") };
-        assert_eq!(lines.len(), 5);
-        assert_eq!(lines[0], "bar!@#$%^&*()[]'\"{}-=`~,.<>/?");
-        assert_eq!(lines[1], "foo");
-        assert_eq!(lines[2], "");
-        assert_eq!(lines[3], "* a");
-        assert_eq!(lines[4], "* b");
+        assert_eq!(lines.len(), 11);
+        assert_eq!(
+            lines[0],
+            TextBlock(vec![Text::from("foo "), Text::new("is not", TextStyle::default().bold()), Text::from(" bar")])
+        );
+        assert_eq!(
+            lines[1],
+            TextBlock(vec![Text::from("![](hehe.png)"), Text::from(" test "), Text::from("![](potato.png)")])
+        );
+        assert_eq!(lines[2], TextBlock::from(""));
+        assert_eq!(lines[3], TextBlock(vec![Text::from("* "), Text::from("a")]));
+        assert_eq!(lines[4], TextBlock(vec![Text::from("* "), Text::from("b")]));
+        assert_eq!(lines[5], TextBlock::from(""));
+        assert_eq!(lines[6], TextBlock(vec![Text::from("1. "), Text::from("a")]));
+        assert_eq!(lines[7], TextBlock(vec![Text::from("2. "), Text::from("b")]));
+        assert_eq!(lines[8], TextBlock::from(""));
+        assert_eq!(lines[9], TextBlock(vec![Text::from("1) "), Text::from("a")]));
+        assert_eq!(lines[10], TextBlock(vec![Text::from("2) "), Text::from("b")]));
     }
 
     #[test]
@@ -779,11 +875,11 @@ foo
         );
         let MarkdownElement::BlockQuote(lines) = parsed else { panic!("not a block quote: {parsed:?}") };
         assert_eq!(lines.len(), 5);
-        assert_eq!(lines[0], "bar");
-        assert_eq!(lines[1], "foo");
-        assert_eq!(lines[2], "");
-        assert_eq!(lines[3], "* a");
-        assert_eq!(lines[4], "* b");
+        assert_eq!(lines[0], TextBlock::from("bar"));
+        assert_eq!(lines[1], TextBlock::from("foo"));
+        assert_eq!(lines[2], TextBlock::from(""));
+        assert_eq!(lines[3], TextBlock(vec![Text::from("* "), Text::from("a")]));
+        assert_eq!(lines[4], TextBlock(vec![Text::from("* "), Text::from("b")]));
     }
 
     #[test]
