@@ -1,6 +1,6 @@
 use crate::{
     custom::OptionsConfig,
-    markdown::text::WeightedTextBlock,
+    markdown::text::{WeightedText, WeightedTextBlock},
     media::image::Image,
     render::properties::WindowSize,
     style::{Color, Colors},
@@ -493,6 +493,18 @@ pub(crate) struct PresentationMetadata {
     #[serde(default)]
     pub(crate) sub_title: Option<String>,
 
+    /// The presentation event.
+    #[serde(default)]
+    pub(crate) event: Option<String>,
+
+    /// The presentation location.
+    #[serde(default)]
+    pub(crate) location: Option<String>,
+
+    /// The presentation date.
+    #[serde(default)]
+    pub(crate) date: Option<String>,
+
     /// The presentation author.
     #[serde(default)]
     pub(crate) author: Option<String>,
@@ -510,6 +522,19 @@ pub(crate) struct PresentationMetadata {
     pub(crate) options: Option<OptionsConfig>,
 }
 
+impl PresentationMetadata {
+    /// Check if this presentation has frontmatter.
+    pub(crate) fn has_frontmatter(&self) -> bool {
+        self.title.is_some()
+            || self.sub_title.is_some()
+            || self.event.is_some()
+            || self.location.is_some()
+            || self.date.is_some()
+            || self.author.is_some()
+            || !self.authors.is_empty()
+    }
+}
+
 /// A presentation's theme metadata.
 #[derive(Clone, Debug, Default, Deserialize)]
 pub(crate) struct PresentationThemeMetadata {
@@ -529,18 +554,15 @@ pub(crate) struct PresentationThemeMetadata {
 /// A line of preformatted text to be rendered.
 #[derive(Clone, Debug, PartialEq)]
 pub(crate) struct BlockLine {
-    pub(crate) text: BlockLineText,
-    pub(crate) unformatted_length: u16,
+    pub(crate) prefix: WeightedText,
+    pub(crate) right_padding_length: u16,
+    pub(crate) repeat_prefix_on_wrap: bool,
+    pub(crate) text: WeightedTextBlock,
     pub(crate) block_length: u16,
+    pub(crate) block_color: Option<Color>,
     pub(crate) alignment: Alignment,
 }
 
-#[derive(Clone, Debug, PartialEq)]
-pub(crate) enum BlockLineText {
-    Preformatted(String),
-    Weighted(WeightedTextBlock),
-}
-
 /// A render operation.
 ///
 /// Render operations are primitives that allow the input markdown file to be decoupled with what
@@ -645,7 +667,9 @@ pub(crate) trait AsRenderOperations: Debug + 'static {
     fn as_render_operations(&self, dimensions: &WindowSize) -> Vec<RenderOperation>;
 
     /// Get the content in this type to diff it against another `AsRenderOperations`.
-    fn diffable_content(&self) -> Option<&str>;
+    fn diffable_content(&self) -> Option<&str> {
+        None
+    }
 }
 
 /// An operation that can be rendered asynchronously.
