@@ -1,19 +1,23 @@
-use super::{code::CodeLine, execution::SnippetExecutionDisabledOperation, modals::KeyBindingsModalBuilder};
+use super::{
+    code::{CodeBlockParser, CodeLine, ExternalFile, Highlight, HighlightGroup, Snippet, SnippetLanguage},
+    execution::{DisplaySeparator, RunAcquireTerminalSnippet, SnippetExecutionDisabledOperation},
+    modals::KeyBindingsModalBuilder,
+};
 use crate::{
     custom::{KeyBindingsConfig, OptionsConfig},
     execute::SnippetExecutor,
     markdown::{
         elements::{
-            Highlight, HighlightGroup, ListItem, ListItemType, MarkdownElement, ParagraphElement, Percent,
-            PercentParseError, Snippet, SnippetLanguage, SourcePosition, Table, TableRow, Text, TextBlock,
+            ListItem, ListItemType, MarkdownElement, Percent, PercentParseError, SourcePosition, Table, TableRow, Text,
+            TextBlock,
         },
         text::WeightedTextBlock,
     },
     media::{image::Image, printer::RegisterImageError, register::ImageRegistry},
     presentation::{
-        BlockLine, BlockLineText, ChunkMutator, ImageProperties, ImageSize, MarginProperties, Modals, Presentation,
-        PresentationMetadata, PresentationState, PresentationThemeMetadata, RenderOperation, Slide, SlideBuilder,
-        SlideChunk,
+        AsRenderOperations, BlockLine, ChunkMutator, ImageProperties, ImageSize, MarginProperties, Modals,
+        Presentation, PresentationMetadata, PresentationState, PresentationThemeMetadata, RenderAsync, RenderOperation,
+        Slide, SlideBuilder, SlideChunk,
     },
     processing::{
         code::{CodePreparer, HighlightContext, HighlightMutator, HighlightedLine},
@@ -22,11 +26,15 @@ use crate::{
         modals::IndexBuilder,
         separator::RenderSeparator,
     },
-    render::highlighting::{CodeHighlighter, HighlightThemeSet},
+    render::{
+        highlighting::{CodeHighlighter, HighlightThemeSet},
+        properties::WindowSize,
+    },
     resource::{LoadImageError, Resources},
     style::{Color, Colors, TextStyle},
     theme::{
-        Alignment, AuthorPositioning, ElementType, LoadThemeError, Margin, PresentationTheme, PresentationThemeSet,
+        Alignment, AuthorPositioning, CodeBlockStyle, ElementType, LoadThemeError, Margin, PresentationTheme,
+        PresentationThemeSet,
     },
     third_party::{ThirdPartyRender, ThirdPartyRenderError, ThirdPartyRenderRequest},
 };
@@ -57,6 +65,7 @@ pub struct PresentationBuilderOptions {
     pub print_modal_background: bool,
     pub strict_front_matter_parsing: bool,
     pub enable_snippet_execution: bool,
+    pub enable_snippet_execution_replace: bool,
 }
 
 impl PresentationBuilderOptions {
@@ -88,6 +97,7 @@ impl Default for PresentationBuilderOptions {
             print_modal_background: false,
             strict_front_matter_parsing: true,
             enable_snippet_execution: false,
+            enable_snippet_execution_replace: false,
         }
     }
 }
@@ -231,7 +241,7 @@ impl<'a> PresentationBuilder<'a> {
     }
 
     fn push_slide_prelude(&mut self) {
-        let colors = self.theme.default_style.colors.clone();
+        let colors = self.theme.default_style.colors;
         self.chunk_operations.extend([
             RenderOperation::SetColors(colors),
             RenderOperation::ClearScreen,
@@ -253,7 +263,7 @@ impl<'a> PresentationBuilder<'a> {
             MarkdownElement::Heading { level, text } => self.push_heading(level, text),
             MarkdownElement::Paragraph(elements) => self.push_paragraph(elements)?,
             MarkdownElement::List(elements) => self.push_list(elements),
-            MarkdownElement::Snippet(code) => self.push_code(code)?,
+            MarkdownElement::Snippet { info, code, source_position } => self.push_code(info, code, source_position)?,
             MarkdownElement::Table(table) => self.push_table(table),
             MarkdownElement::ThematicBreak => self.process_thematic_break(),
             MarkdownElement::Comment { comment, source_position } => self.process_comment(comment, source_position)?,
@@ -281,13 +291,19 @@ impl<'a> PresentationBuilder<'a> {
         if let Some(options) = metadata.options.take() {
             self.options.merge(options);
         }
-        self.footer_context.borrow_mut().author = metadata.author.clone().unwrap_or_default();
-        self.set_theme(&metadata.theme)?;
-        if metadata.title.is_some()
-            || metadata.sub_title.is_some()
-            || metadata.author.is_some()
-            || !metadata.authors.is_empty()
+
         {
+            let mut footer_context = self.footer_context.borrow_mut();
+            footer_context.title = metadata.title.clone().unwrap_or_default();
+            footer_context.sub_title = metadata.sub_title.clone().unwrap_or_default();
+            footer_context.location = metadata.location.clone().unwrap_or_default();
+            footer_context.event = metadata.event.clone().unwrap_or_default();
+            footer_context.date = metadata.date.clone().unwrap_or_default();
+            footer_context.author = metadata.author.clone().unwrap_or_default();
+        }
+
+        self.set_theme(&metadata.theme)?;
+        if metadata.has_frontmatter() {
             self.push_slide_prelude();
             self.push_intro_slide(metadata);
         }
@@ -336,30 +352,42 @@ impl<'a> PresentationBuilder<'a> {
     }
 
     fn push_intro_slide(&mut self, metadata: PresentationMetadata) {
-        let styles = &self.theme.intro_slide;
-        let title = Text::new(
-            metadata.title.unwrap_or_default().clone(),
-            TextStyle::default().bold().colors(styles.title.colors.clone()),
-        );
-        let sub_title = metadata
-            .sub_title
-            .as_ref()
-            .map(|text| Text::new(text.clone(), TextStyle::default().colors(styles.subtitle.colors.clone())));
+        let styles = self.theme.intro_slide.clone();
+        let create_text =
+            |text: Option<String>, style: TextStyle| -> Option<Text> { text.map(|text| Text::new(text, style)) };
+        let title = create_text(metadata.title, TextStyle::default().bold().colors(styles.title.colors));
+        let sub_title = create_text(metadata.sub_title, TextStyle::default().colors(styles.subtitle.colors));
+        let event = create_text(metadata.event, TextStyle::default().colors(styles.event.colors));
+        let location = create_text(metadata.location, TextStyle::default().colors(styles.location.colors));
+        let date = create_text(metadata.date, TextStyle::default().colors(styles.date.colors));
         let authors: Vec<_> = metadata
             .author
             .into_iter()
             .chain(metadata.authors)
-            .map(|author| Text::new(author, TextStyle::default().colors(styles.author.colors.clone())))
+            .map(|author| Text::new(author, TextStyle::default().colors(styles.author.colors)))
             .collect();
         if styles.footer == Some(false) {
             self.slide_state.ignore_footer = true;
         }
         self.chunk_operations.push(RenderOperation::JumpToVerticalCenter);
-        self.push_text(TextBlock::from(title), ElementType::PresentationTitle);
-        self.push_line_break();
-        if let Some(text) = sub_title {
-            self.push_text(TextBlock::from(text), ElementType::PresentationSubTitle);
+        if let Some(title) = title {
+            self.push_line(title, ElementType::PresentationTitle);
+        }
+        if let Some(sub_title) = sub_title {
+            self.push_line(sub_title, ElementType::PresentationSubTitle);
+        }
+        if event.is_some() || location.is_some() || date.is_some() {
             self.push_line_break();
+            self.push_line_break();
+            if let Some(event) = event {
+                self.push_line(event, ElementType::PresentationEvent);
+            }
+            if let Some(location) = location {
+                self.push_line(location, ElementType::PresentationLocation);
+            }
+            if let Some(date) = date {
+                self.push_line(date, ElementType::PresentationDate);
+            }
         }
         if !authors.is_empty() {
             match self.theme.intro_slide.author.positioning {
@@ -373,8 +401,7 @@ impl<'a> PresentationBuilder<'a> {
                 }
             };
             for author in authors {
-                self.push_text(TextBlock::from(author), ElementType::PresentationAuthor);
-                self.push_line_break();
+                self.push_line(author, ElementType::PresentationAuthor);
             }
         }
         self.slide_state.title = Some(TextBlock::from("[Introduction]"));
@@ -477,7 +504,7 @@ impl<'a> PresentationBuilder<'a> {
         }
 
         let style = self.theme.slide_title.clone();
-        let mut text_style = TextStyle::default().colors(style.colors.clone());
+        let mut text_style = TextStyle::default().colors(style.colors);
         if style.bold.unwrap_or_default() {
             text_style = text_style.bold();
         }
@@ -520,24 +547,17 @@ impl<'a> PresentationBuilder<'a> {
             prefix.push(' ');
             text.0.insert(0, Text::from(prefix));
         }
-        let text_style = TextStyle::default().bold().colors(style.colors.clone());
+        let text_style = TextStyle::default().bold().colors(style.colors);
         text.apply_style(&text_style);
 
         self.push_text(text, element_type);
         self.push_line_break();
     }
 
-    fn push_paragraph(&mut self, elements: Vec<ParagraphElement>) -> Result<(), BuildError> {
-        for element in elements {
-            match element {
-                ParagraphElement::Text(text) => {
-                    self.push_text(text, ElementType::Paragraph);
-                    self.push_line_break();
-                }
-                ParagraphElement::LineBreak => {
-                    // Line breaks are already pushed after every text chunk.
-                }
-            };
+    fn push_paragraph(&mut self, lines: Vec<TextBlock>) -> Result<(), BuildError> {
+        for text in lines {
+            self.push_text(text, ElementType::Paragraph);
+            self.push_line_break();
         }
         Ok(())
     }
@@ -575,7 +595,7 @@ impl<'a> PresentationBuilder<'a> {
         };
         self.chunk_operations.extend([
             RenderOperation::RenderImage(image, properties),
-            RenderOperation::SetColors(self.theme.default_style.colors.clone()),
+            RenderOperation::SetColors(self.theme.default_style.colors),
         ]);
         Ok(())
     }
@@ -639,7 +659,7 @@ impl<'a> PresentationBuilder<'a> {
         }
     }
 
-    fn push_block_quote(&mut self, lines: Vec<String>) {
+    fn push_block_quote(&mut self, lines: Vec<TextBlock>) {
         let prefix = self.theme.block_quote.prefix.clone().unwrap_or_default();
         let block_length = lines.iter().map(|line| line.width() + prefix.width()).max().unwrap_or(0) as u16;
         let prefix_color = self.theme.block_quote.colors.prefix.or(self.theme.block_quote.colors.base.foreground);
@@ -649,26 +669,32 @@ impl<'a> PresentationBuilder<'a> {
                 .colors(Colors { foreground: prefix_color, background: self.theme.block_quote.colors.base.background }),
         );
         let alignment = self.theme.alignment(&ElementType::BlockQuote).clone();
-        let style = TextStyle::default().colors(self.theme.block_quote.colors.base.clone());
 
-        for line in lines {
-            let line = TextBlock(vec![prefix.clone(), Text::new(line, style.clone())]);
-            self.chunk_operations.extend([
-                // Print a preformatted empty block so we fill in the line with properly colored
-                // spaces.
-                RenderOperation::SetColors(self.theme.block_quote.colors.base.clone()),
-                RenderOperation::RenderBlockLine(BlockLine {
-                    text: BlockLineText::Preformatted("".into()),
-                    unformatted_length: 0,
-                    block_length,
-                    alignment: alignment.clone(),
-                }),
-                // Now render our prefix + entire line
-                RenderOperation::RenderText { line: line.into(), alignment: alignment.clone() },
-            ]);
+        for mut line in lines {
+            // Apply our colors to each chunk in this line.
+            for text in &mut line.0 {
+                text.style.colors = self.theme.block_quote.colors.base;
+                if text.style.is_code() {
+                    text.style.colors = self.theme.inline_code.colors;
+                }
+            }
+            self.chunk_operations.push(RenderOperation::RenderBlockLine(BlockLine {
+                prefix: prefix.clone().into(),
+                right_padding_length: 0,
+                repeat_prefix_on_wrap: true,
+                text: line.into(),
+                block_length,
+                alignment: alignment.clone(),
+                block_color: self.theme.block_quote.colors.base.background,
+            }));
             self.push_line_break();
         }
-        self.chunk_operations.push(RenderOperation::SetColors(self.theme.default_style.colors.clone()));
+        self.chunk_operations.push(RenderOperation::SetColors(self.theme.default_style.colors));
+    }
+
+    fn push_line(&mut self, text: Text, element_type: ElementType) {
+        self.push_text(TextBlock::from(text), element_type);
+        self.push_line_break();
     }
 
     fn push_text(&mut self, text: TextBlock, element_type: ElementType) {
@@ -679,7 +705,7 @@ impl<'a> PresentationBuilder<'a> {
     fn push_aligned_text(&mut self, mut block: TextBlock, alignment: Alignment) {
         for chunk in &mut block.0 {
             if chunk.style.is_code() {
-                chunk.style.colors = self.theme.inline_code.colors.clone();
+                chunk.style.colors = self.theme.inline_code.colors;
             }
         }
         if !block.0.is_empty() {
@@ -694,26 +720,49 @@ impl<'a> PresentationBuilder<'a> {
         self.chunk_operations.push(RenderOperation::RenderLineBreak);
     }
 
-    fn push_code(&mut self, code: Snippet) -> Result<(), BuildError> {
-        if code.attributes.auto_render {
-            return self.push_rendered_code(code);
+    fn push_differ(&mut self, text: String) {
+        self.chunk_operations.push(RenderOperation::RenderDynamic(Rc::new(Differ(text))));
+    }
+
+    fn push_code(&mut self, info: String, code: String, source_position: SourcePosition) -> Result<(), BuildError> {
+        let mut snippet = CodeBlockParser::parse(info, code)
+            .map_err(|e| BuildError::InvalidCode { line: source_position.start.line + 1, error: e.to_string() })?;
+        if matches!(snippet.language, SnippetLanguage::File) {
+            snippet = self.load_external_snippet(snippet, source_position.clone())?;
         }
-        let lines = CodePreparer::new(&self.theme).prepare(&code);
+        self.push_differ(snippet.contents.clone());
+
+        if snippet.attributes.auto_render {
+            return self.push_rendered_code(snippet, source_position);
+        } else if snippet.attributes.execute_replace && self.options.enable_snippet_execution_replace {
+            return self.push_code_execution(snippet, 0, ExecutionMode::ReplaceSnippet);
+        }
+        let lines =
+            CodePreparer::new(&self.theme, self.code_executor.hidden_line_prefix(&snippet.language)).prepare(&snippet);
         let block_length = lines.iter().map(|line| line.width()).max().unwrap_or(0);
-        let (lines, context) = self.highlight_lines(&code, lines, block_length);
+        let (lines, context) = self.highlight_lines(&snippet, lines, block_length);
         for line in lines {
             self.chunk_operations.push(RenderOperation::RenderDynamic(Rc::new(line)));
         }
-        self.chunk_operations.push(RenderOperation::SetColors(self.theme.default_style.colors.clone()));
+        self.chunk_operations.push(RenderOperation::SetColors(self.theme.default_style.colors));
         if self.options.allow_mutations && context.borrow().groups.len() > 1 {
             self.chunk_mutators.push(Box::new(HighlightMutator::new(context)));
         }
-        if code.attributes.execute {
+
+        if snippet.attributes.execute_replace && !self.options.enable_snippet_execution_replace {
+            let operation = SnippetExecutionDisabledOperation::new(
+                self.theme.execution_output.status.failure,
+                self.theme.code.alignment.clone().unwrap_or_default(),
+            );
+            operation.start_render();
+            self.chunk_operations.push(RenderOperation::RenderDynamic(Rc::new(operation)))
+        }
+        if snippet.attributes.execute {
             if self.options.enable_snippet_execution {
-                self.push_code_execution(code, block_length)?;
+                self.push_code_execution(snippet, block_length, ExecutionMode::AlongSnippet)?;
             } else {
                 let operation = SnippetExecutionDisabledOperation::new(
-                    self.theme.execution_output.status.failure.clone(),
+                    self.theme.execution_output.status.failure,
                     self.theme.code.alignment.clone().unwrap_or_default(),
                 );
                 self.chunk_operations.push(RenderOperation::RenderAsync(Rc::new(operation)))
@@ -722,14 +771,39 @@ impl<'a> PresentationBuilder<'a> {
         Ok(())
     }
 
-    fn push_rendered_code(&mut self, code: Snippet) -> Result<(), BuildError> {
+    fn load_external_snippet(
+        &mut self,
+        mut code: Snippet,
+        source_position: SourcePosition,
+    ) -> Result<Snippet, BuildError> {
+        // TODO clean up this repeated thing
+        let line = source_position.start.line + 1;
+        let file: ExternalFile =
+            serde_yaml::from_str(&code.contents).map_err(|e| BuildError::InvalidCode { line, error: e.to_string() })?;
+        let path = file.path;
+        let path_display = path.display();
+        let contents = self
+            .resources
+            .external_snippet(&path)
+            .map_err(|e| BuildError::InvalidCode { line, error: format!("failed to load {path_display}: {e}") })?;
+        code.language = file.language;
+        code.contents = contents;
+        Ok(code)
+    }
+
+    fn push_rendered_code(&mut self, code: Snippet, source_position: SourcePosition) -> Result<(), BuildError> {
         let Snippet { contents, language, attributes } = code;
         let error_holder = self.presentation_state.async_error_holder();
         let request = match language {
             SnippetLanguage::Typst => ThirdPartyRenderRequest::Typst(contents, self.theme.typst.clone()),
             SnippetLanguage::Latex => ThirdPartyRenderRequest::Latex(contents, self.theme.typst.clone()),
             SnippetLanguage::Mermaid => ThirdPartyRenderRequest::Mermaid(contents, self.theme.mermaid.clone()),
-            _ => panic!("language {language:?} should not be renderable"),
+            _ => {
+                return Err(BuildError::InvalidCode {
+                    line: source_position.start.line + 1,
+                    error: format!("language {language:?} doesn't support rendering"),
+                })?;
+            }
         };
         let operation =
             self.third_party.render(request, &self.theme, error_holder, self.slides.len() + 1, attributes.width)?;
@@ -744,9 +818,10 @@ impl<'a> PresentationBuilder<'a> {
         block_length: usize,
     ) -> (Vec<HighlightedLine>, Rc<RefCell<HighlightContext>>) {
         let mut code_highlighter = self.highlighter.language_highlighter(&code.language);
+        let style = self.code_style(code);
         let dim_style = {
             let mut highlighter = self.highlighter.language_highlighter(&SnippetLanguage::Rust);
-            highlighter.style_line("//").next().expect("no styles").style
+            highlighter.style_line("//", &style).0.first().expect("no styles").style
         };
         let groups = match self.options.allow_mutations {
             true => code.attributes.highlight_groups.clone(),
@@ -756,27 +831,86 @@ impl<'a> PresentationBuilder<'a> {
             groups,
             current: 0,
             block_length,
-            alignment: self.theme.alignment(&ElementType::Code),
+            alignment: style.alignment.clone().unwrap_or_default(),
         }));
 
         let mut output = Vec::new();
-        let block_style = &self.theme.code;
         for line in lines.into_iter() {
-            let highlighted = line.highlight(&dim_style, &mut code_highlighter, block_style);
-            let not_highlighted = line.dim(&dim_style, block_style);
-            let width = line.width();
+            let prefix = line.dim_prefix(&dim_style);
+            let highlighted = line.highlight(&mut code_highlighter, &style);
+            let not_highlighted = line.dim(&dim_style);
             let line_number = line.line_number;
             let context = context.clone();
-            output.push(HighlightedLine { highlighted, not_highlighted, line_number, width, context });
+            output.push(HighlightedLine {
+                prefix,
+                right_padding_length: line.right_padding_length,
+                highlighted,
+                not_highlighted,
+                line_number,
+                context,
+                block_color: dim_style.colors.background,
+            });
         }
         (output, context)
     }
 
-    fn push_code_execution(&mut self, code: Snippet, block_length: usize) -> Result<(), BuildError> {
+    fn code_style(&self, snippet: &Snippet) -> CodeBlockStyle {
+        let mut style = self.theme.code.clone();
+        if snippet.attributes.no_background {
+            style.alignment = match style.alignment {
+                Some(Alignment::Center { .. }) => {
+                    Some(Alignment::Center { minimum_size: 0, minimum_margin: Margin::default() })
+                }
+                Some(Alignment::Left { .. }) => Some(Alignment::Left { margin: Margin::default() }),
+                Some(Alignment::Right { .. }) => Some(Alignment::Right { margin: Margin::default() }),
+                None => None,
+            };
+            style.background = Some(false);
+        }
+        style
+    }
+
+    fn push_code_execution(
+        &mut self,
+        code: Snippet,
+        block_length: usize,
+        mode: ExecutionMode,
+    ) -> Result<(), BuildError> {
         if !self.code_executor.is_execution_supported(&code.language) {
             return Err(BuildError::UnsupportedExecution(code.language));
         }
-        let operation = RunSnippetOperation::new(code, self.code_executor.clone(), &self.theme, block_length as u16);
+        if code.attributes.acquire_terminal {
+            let block_length = block_length as u16;
+            let block_length = match self.theme.code.alignment.clone().unwrap_or_default() {
+                Alignment::Left { .. } | Alignment::Right { .. } => block_length,
+                Alignment::Center { minimum_size, .. } => block_length.max(minimum_size),
+            };
+            let operation = RunAcquireTerminalSnippet::new(
+                code,
+                self.code_executor.clone(),
+                self.theme.execution_output.status.clone(),
+                block_length,
+            );
+            let operation = RenderOperation::RenderAsync(Rc::new(operation));
+            self.chunk_operations.push(operation);
+            return Ok(());
+        }
+        let separator = match mode {
+            ExecutionMode::AlongSnippet => DisplaySeparator::On,
+            ExecutionMode::ReplaceSnippet => DisplaySeparator::Off,
+        };
+        let alignment = self.code_style(&code).alignment.unwrap_or_default();
+        let operation = RunSnippetOperation::new(
+            code,
+            self.code_executor.clone(),
+            &self.theme,
+            block_length as u16,
+            separator,
+            alignment,
+        );
+        if matches!(mode, ExecutionMode::ReplaceSnippet) {
+            operation.start_render();
+        }
         let operation = RenderOperation::RenderAsync(Rc::new(operation));
         self.chunk_operations.push(operation);
         Ok(())
@@ -950,6 +1084,9 @@ pub enum BuildError {
     #[error("invalid theme: {0}")]
     InvalidTheme(#[from] LoadThemeError),
 
+    #[error("invalid code at line {line}: {error}")]
+    InvalidCode { line: usize, error: String },
+
     #[error("invalid code highlighter theme: '{0}'")]
     InvalidCodeTheme(String),
 
@@ -981,6 +1118,11 @@ pub enum BuildError {
     UnsupportedExecution(SnippetLanguage),
 }
 
+enum ExecutionMode {
+    AlongSnippet,
+    ReplaceSnippet,
+}
+
 #[derive(Debug, Clone, PartialEq, Deserialize)]
 #[serde(rename_all = "snake_case")]
 enum CommentCommand {
@@ -1084,6 +1226,15 @@ struct StrictPresentationMetadata {
     #[serde(default)]
     sub_title: Option<String>,
 
+    #[serde(default)]
+    event: Option<String>,
+
+    #[serde(default)]
+    location: Option<String>,
+
+    #[serde(default)]
+    date: Option<String>,
+
     #[serde(default)]
     author: Option<String>,
 
@@ -1099,8 +1250,9 @@ struct StrictPresentationMetadata {
 
 impl From<StrictPresentationMetadata> for PresentationMetadata {
     fn from(strict: StrictPresentationMetadata) -> Self {
-        let StrictPresentationMetadata { title, sub_title, author, authors, theme, options } = strict;
-        Self { title, sub_title, author, authors, theme, options }
+        let StrictPresentationMetadata { title, sub_title, event, location, date, author, authors, theme, options } =
+            strict;
+        Self { title, sub_title, event, location, date, author, authors, theme, options }
     }
 }
 
@@ -1121,10 +1273,22 @@ struct ImageAttributes {
     width: Option<Percent>,
 }
 
+#[derive(Debug)]
+struct Differ(String);
+
+impl AsRenderOperations for Differ {
+    fn as_render_operations(&self, _: &WindowSize) -> Vec<RenderOperation> {
+        Vec::new()
+    }
+
+    fn diffable_content(&self) -> Option<&str> {
+        Some(&self.0)
+    }
+}
+
 #[cfg(test)]
 mod test {
     use super::*;
-    use crate::markdown::elements::SnippetAttributes;
     use rstest::rstest;
 
     fn build_presentation(elements: Vec<MarkdownElement>) -> Presentation {
@@ -1391,10 +1555,11 @@ mod test {
     #[test]
     fn iterate_list_starting_from_other() {
         let list = ListIterator::new(
-            vec![
-                ListItem { depth: 0, contents: "0".into(), item_type: ListItemType::Unordered },
-                ListItem { depth: 0, contents: "1".into(), item_type: ListItemType::Unordered },
-            ],
+            vec![ListItem { depth: 0, contents: "0".into(), item_type: ListItemType::Unordered }, ListItem {
+                depth: 0,
+                contents: "1".into(),
+                item_type: ListItemType::Unordered,
+            }],
             3,
         );
         let expected_indexes = [3, 4];
@@ -1481,10 +1646,10 @@ mod test {
 
     #[test]
     fn implicit_slide_ends_with_front_matter() {
-        let elements = vec![
-            MarkdownElement::FrontMatter("theme:\n name: light".into()),
-            MarkdownElement::SetexHeading { text: "hi".into() },
-        ];
+        let elements =
+            vec![MarkdownElement::FrontMatter("theme:\n name: light".into()), MarkdownElement::SetexHeading {
+                text: "hi".into(),
+            }];
         let options = PresentationBuilderOptions { implicit_slide_ends: true, ..Default::default() };
         let slides = build_presentation_with_options(elements, options).into_slides();
         assert_eq!(slides.len(), 1);
@@ -1524,7 +1689,7 @@ mod test {
         let elements = vec![
             MarkdownElement::Paragraph(vec![]),
             MarkdownElement::ThematicBreak,
-            MarkdownElement::Paragraph(vec![ParagraphElement::Text("hi".into())]),
+            MarkdownElement::Paragraph(vec!["hi".into()]),
         ];
         let presentation = build_presentation_with_options(elements, options);
         assert_eq!(presentation.iter_slides().count(), 2);
@@ -1548,30 +1713,27 @@ mod test {
     #[case::enabled(true)]
     #[case::disabled(false)]
     fn snippet_execution(#[case] enabled: bool) {
-        let element = MarkdownElement::Snippet(Snippet {
-            contents: "".into(),
-            language: SnippetLanguage::Rust,
-            attributes: SnippetAttributes { execute: true, ..Default::default() },
-        });
+        let element = MarkdownElement::Snippet {
+            info: "rust +exec".into(),
+            code: "".into(),
+            source_position: Default::default(),
+        };
         let options = PresentationBuilderOptions { enable_snippet_execution: enabled, ..Default::default() };
         let presentation = build_presentation_with_options(vec![element], options);
         let slide = presentation.iter_slides().next().unwrap();
         let mut found_render_block = false;
         let mut found_cant_render_block = false;
         for operation in slide.iter_visible_operations() {
-            match operation {
-                RenderOperation::RenderAsync(operation) => {
-                    let operation = format!("{operation:?}");
-                    if operation.contains("RunSnippetOperation") {
-                        assert!(enabled);
-                        found_render_block = true;
-                    } else if operation.contains("SnippetExecutionDisabledOperation") {
-                        assert!(!enabled);
-                        found_cant_render_block = true;
-                    }
+            if let RenderOperation::RenderAsync(operation) = operation {
+                let operation = format!("{operation:?}");
+                if operation.contains("RunSnippetOperation") {
+                    assert!(enabled);
+                    found_render_block = true;
+                } else if operation.contains("SnippetExecutionDisabledOperation") {
+                    assert!(!enabled);
+                    found_cant_render_block = true;
                 }
-                _ => (),
-            };
+            }
         }
         if found_render_block {
             assert!(enabled, "snippet execution block found but not enabled");
@@ -1592,7 +1754,7 @@ mod test {
     #[case::no_prefix("width", None)]
     fn image_attributes(#[case] input: &str, #[case] expectation: Option<u8>) {
         let attributes =
-            PresentationBuilder::parse_image_attributes(&input, "image:", Default::default()).expect("failed to parse");
+            PresentationBuilder::parse_image_attributes(input, "image:", Default::default()).expect("failed to parse");
         assert_eq!(attributes.width, expectation.map(Percent));
     }
 
@@ -1604,4 +1766,21 @@ mod test {
             PresentationBuilder::parse_image_attributes(input, "", Default::default()).expect("failed to parse");
         assert_eq!(attributes.width, expectation.map(Percent));
     }
+
+    #[test]
+    fn external_snippet() {
+        let temp = tempfile::NamedTempFile::new().expect("failed to create tempfile");
+        let path = temp.path().file_name().expect("no file name").to_string_lossy();
+        let code = format!(
+            r"
+path: {path}
+language: rust"
+        );
+        let elements = vec![MarkdownElement::Snippet {
+            info: "file +line_numbers +exec".into(),
+            code,
+            source_position: Default::default(),
+        }];
+        build_presentation(elements);
+    }
 }
