@@ -206,6 +206,9 @@ impl PresentationTheme {
             Code => &self.code.alignment,
             PresentationTitle => &self.intro_slide.title.alignment,
             PresentationSubTitle => &self.intro_slide.subtitle.alignment,
+            PresentationEvent => &self.intro_slide.event.alignment,
+            PresentationLocation => &self.intro_slide.location.alignment,
+            PresentationDate => &self.intro_slide.date.alignment,
             PresentationAuthor => &self.intro_slide.author.alignment,
             Table => &self.table,
             BlockQuote => &self.block_quote.alignment,
@@ -336,6 +339,18 @@ pub(crate) struct IntroSlideStyle {
     #[serde(default)]
     pub(crate) subtitle: BasicStyle,
 
+    /// The style of the event line.
+    #[serde(default)]
+    pub(crate) event: BasicStyle,
+
+    /// The style of the location line.
+    #[serde(default)]
+    pub(crate) location: BasicStyle,
+
+    /// The style of the date line.
+    #[serde(default)]
+    pub(crate) date: BasicStyle,
+
     /// The style of the author line.
     #[serde(default)]
     pub(crate) author: AuthorStyle,
@@ -459,12 +474,7 @@ pub(crate) enum FooterStyle {
 
 impl Default for FooterStyle {
     fn default() -> Self {
-        Self::Template {
-            left: Some("{current_slide} / {total_slides}".to_string()),
-            center: None,
-            right: None,
-            colors: Colors::default(),
-        }
+        Self::Template { left: None, center: None, right: None, colors: Colors::default() }
     }
 }
 
@@ -513,6 +523,10 @@ pub(crate) struct ExecutionStatusBlockStyle {
     /// The colors for the "finished with error" status.
     #[serde(default)]
     pub(crate) failure: Colors,
+
+    /// The colors for the "not started" status.
+    #[serde(default)]
+    pub(crate) not_started: Colors,
 }
 
 /// The style for inline code.
@@ -556,6 +570,10 @@ impl Margin {
             }
         }
     }
+
+    pub(crate) fn is_empty(&self) -> bool {
+        matches!(self, Self::Fixed(0) | Self::Percent(0))
+    }
 }
 
 impl Default for Margin {
@@ -580,6 +598,9 @@ pub(crate) enum ElementType {
     Code,
     PresentationTitle,
     PresentationSubTitle,
+    PresentationEvent,
+    PresentationLocation,
+    PresentationDate,
     PresentationAuthor,
     Table,
     BlockQuote,
@@ -597,7 +618,7 @@ pub(crate) enum AuthorPositioning {
     PageBottom,
 }
 
-/// Where to position the author's name in the intro slide.
+/// Typst styles.
 #[derive(Clone, Debug, Default, Deserialize, Serialize)]
 pub(crate) struct TypstStyle {
     /// The horizontal margin on the generated images.
@@ -611,7 +632,7 @@ pub(crate) struct TypstStyle {
     pub(crate) colors: Colors,
 }
 
-/// Where to position the author's name in the intro slide.
+/// Mermaid styles.
 #[derive(Clone, Debug, Default, Deserialize, Serialize)]
 pub(crate) struct MermaidStyle {
     /// The mermaidjs theme to use.
@@ -655,7 +676,7 @@ pub enum LoadThemeError {
 #[cfg(test)]
 mod test {
     use super::*;
-    use tempfile::{tempdir, TempDir};
+    use tempfile::{TempDir, tempdir};
 
     fn write_theme(name: &str, theme: PresentationTheme, directory: &TempDir) {
         let theme = serde_yaml::to_string(&theme).unwrap();
